import pickle
import os
import sys
sys.path.append('../utils')
import datasets
import clusout
from experiment import Experiment


DATASETS = datasets.vens_datasets[-2:]
FOLDS = 10
EXPERIMENTS = [
    Experiment(clustering='PCT_standard', name='standard', aggregation='None'),
    Experiment(clustering='nickel_d25_i100', dim=25, epochs=100, name='nickel')
]
comb_times = {'PCT_standard.npy': 0}
columns = ['dataset', 'clustering', 'aggregation', 'dim', 'precision', 'AUROC', 'AUPRC', '$AU\overline{PRC}$', 'time', 'fold', 'epochs', 'trees', 'checkpoint']
out_lines = [','.join(columns)]
not_finished = 0
for dset in DATASETS:
    embeddng_times = pickle.load(open('../data/{}.times'.format(dset.path), 'rb'))
    embeddng_times.update(comb_times)
    for exp in EXPERIMENTS:
        if exp.name == 'standard':
            exp.dim = dset.num_labels

        for fold in range(10):
            path = 'BudgetJobs/{}/{}/fold{}/results/settings.out'.format(exp.clustering, dset.path[:dset.path.rfind('/')], fold)
            if os.path.exists(path) and os.stat(path).st_size > 42:
                with open(path) as f:
                    outlines = f.readlines()
                try:
                    num_trees = clusout.get_tree_numbers(outlines)
                    for i, ntrees in enumerate(num_trees):
                        auroc = clusout.get_error(outlines, ['Testing error', 'Forest with {}'.format(ntrees), 'Average AUROC'])
                        auprc = clusout.get_error(outlines, ['Testing error', 'Forest with {}'.format(ntrees), 'Average AUPRC'])
                        pooled = clusout.get_error(outlines, ['Testing error', 'Forest with {}'.format(ntrees), 'Pooled AUPRC'])
                        l = clusout.find_line(outlines, ['Statistics', 'Induction Time:'])
                        induction_time = min(3600, float(l.split()[2]) * ntrees / num_trees[-1])
                        total_time = induction_time + embeddng_times[exp.clustering+'.npy']

                        out_lines.append(','.join([
                            dset.name,
                            exp.name,
                            exp.aggregation,
                            str(exp.dim),
                            str(exp.precision),
                            str(auroc),
                            str(auprc),
                            str(pooled),
                            str(total_time),
                            str(fold),
                            str(exp.epochs),
                            str(ntrees),
                            str(i)
                        ]))
                except IndexError:
                    not_finished += 1
                    print(path)
                    print('Something went wrong!')
            else:
                not_finished += 1
                print(path)
                print('Missing!')

with open('results_budget.csv', 'w') as f:
    print('\n'.join(out_lines), file=f)

print(not_finished)
