import pickle
import os
import sys
sys.path.append('../utils')
import datasets
import clusout
from experiment import Experiment


DATASETS = datasets.vens_datasets[:-2]
AGGREGATIONS = ['mean', 'medoid_eucl', 'medoid_hyper']
FOLDS = 10

for agg in AGGREGATIONS:
    experiments = [
        Experiment(clustering='PCT_standard', name='standard', aggregation='None'),
        Experiment(clustering='PCT_random', name='random', method='ExtraTrees', aggregation='None')
    ]
    comb_times = {'PCT_standard.npy': 0, 'PCT_random.npy': 0}

    for dim in [2, 5, 10, 25, 50, 100]:
        for epochs in [100]:
            experiments.append(Experiment(clustering='nickel_d{}_i{}'.format(dim, epochs),
                                          dim=dim, epochs=epochs, name='nickel', aggregation=agg))

    columns = ['dataset', 'clustering', 'aggregation', 'dim', 'precision', 'AUROC', 'AUPRC', '$AU\overline{PRC}$', 'time', 'fold', 'epochs']
    out_lines = [','.join(columns)]
    not_finished = 0
    for dset in DATASETS:
        embeddng_times = pickle.load(open('../data/{}.times'.format(dset.path), 'rb'))
        embeddng_times.update(comb_times)
        for exp in experiments:
            if exp.name == 'standard':
                exp.dim = dset.num_labels

            for fold in range(10):
                path = 'Jobs_{}/{}/{}/fold{}/results/settings.out'.format(agg, exp.clustering, dset.path[:dset.path.rfind('/')], fold)
                if os.path.exists(path) and os.stat(path).st_size > 42:
                    with open(path) as f:
                        outlines = f.readlines()
                    try:
                        auroc = clusout.get_error(outlines, ['Testing error', 'Forest with', 'Average AUROC'])
                        auprc = clusout.get_error(outlines, ['Testing error', 'Forest with', 'Average AUPRC'])
                        pooled = clusout.get_error(outlines, ['Testing error', 'Forest with', 'Pooled AUPRC'])
                        l = clusout.find_line(outlines, ['Statistics', 'Induction Time:'])
                        induction_time = float(l.split()[2])
                        total_time = induction_time + embeddng_times[exp.clustering+'.npy']

                        out_lines.append(','.join([
                            dset.name,
                            exp.name,
                            exp.aggregation,
                            str(exp.dim),
                            str(exp.precision),
                            str(auroc),
                            str(auprc),
                            str(pooled),
                            str(total_time),
                            str(fold),
                            str(exp.epochs)
                        ]))
                    except IndexError:
                        not_finished += 1
                        print(path)
                        print('Something went wrong!')
                else:
                    not_finished += 1
                    print(path)
                    print('Missing!')

    with open('results_{}.csv'.format(agg), 'w') as f:
        print('\n'.join(out_lines), file=f)

    print(agg, not_finished)
