import subprocess
import sys
sys.path.append('../utils')
import datasets
from experiment import Experiment


XRSL_TEMPLATE = """&
(executable = "run.sh")
(inputFiles =
    ("run.sh" "")
    ("clus.jar" "../../../../../clus.jar")
    ("augmenter.py" "../../../../../../utils/augmenter.py")
    ("hierarchy.py" "../../../../../../utils/hierarchy.py")
    ("arff_manipulation.py" "../../../../../../utils/arff_manipulation.py")
    ("data.arff" "../../../../../../data/{0}")
    ("embedding.npy" "../../../../../../data/{1}")
    ("hierarchy.labels" "../../../../../../data/{2}")
    ("hierarchy.edges" "../../../../../../data/{3}")
    ("settings.s" "")
)
(outputFiles =
    ("settings.out" "")
)
(jobname = "{4}")
(cache = "no")
(count = 4)
(countpernode = 4)
(memory = 8000)
(cputime = "330 hours")
(join = "yes")
(stdout = "{4}.log")
"""


SETTINGS_TEMPLATE = """
[General]
[General]
Verbose = 0
RandomSeed = 42

[Hierarchical]
Type = {0}
WType = ExpAvgParentWeight
HSeparator = /

[Data]
File = data.arff.fold{1}.train
TestSet = data.arff.fold{1}.test

[Attributes]
{2}

[Ensemble]
EnsembleMethod = {3}
Optimize = Yes
Iterations = [5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250]
NumberOfThreads = 4
TimeBudget = 60
SelectRandomSubspaces = 1
EnsembleBootstrapping = Yes
"""

DATASETS = datasets.vens_datasets[-2:]
FOLDS = 10

EXPERIMENTS = [
    Experiment(clustering='PCT_standard', name='standard'),
    Experiment(clustering='nickel_d25_i100', dim=25, epochs=100, name='nickel')
]

for dset in DATASETS:
    for exp in EXPERIMENTS:
        for fold in range(FOLDS):
            attributes = []
            current_index = 1
            if exp.dim > 0:
                attributes.append('Clustering = {}-{}'.format(current_index, current_index+exp.dim-1))
                current_index += exp.dim
            if dset.num_keys > 0:
                attributes.append('Key = {}'.format(','.join([str(x) for x in range(1, dset.num_keys+1)])))
                current_index += dset.num_keys
            attributes.append('Descriptive = {}-{}'.format(current_index, current_index+dset.num_attributes-1))
            current_index += dset.num_attributes
            attributes.append('Target = {}'.format(current_index))

            job_path = 'BudgetJobs/{}/{}/fold{}'.format(exp.clustering, dset.path[:dset.path.rfind('/')], fold)
            data = '{}.joined.arff'.format(dset.path)

            labels = '{}.labels'.format(dset.path)
            edges = '{}.edges'.format(dset.path)
            if exp.dim == 0:
                embedding = labels
            else:
                embedding = '{}.{}.npy'.format(dset.path, exp.clustering)

            subprocess.call('mkdir -p {}'.format(job_path), shell=True)
            with open('{}/settings.s'.format(job_path), 'w') as f:
                print(SETTINGS_TEMPLATE.format(dset.hierarchy_type, fold, '\n'.join(attributes), exp.method), file=f)

            with open('{}/job.xrsl'.format(job_path), 'w') as f:
                print(XRSL_TEMPLATE.format(data, embedding, labels, edges, job_path.replace('/', '-')), file=f)

            with open('{}/run.sh'.format(job_path), 'w') as f:
                print('python augmenter.py data.arff embedding.npy hierarchy.edges hierarchy.labels', file=f)
                print('python arff_manipulation.py data.arff {}'.format(fold), file=f)
                print('java -Xmx50g -jar clus.jar -forest settings.s', file=f)
                print('rm data.arff.*', file=f)
