import subprocess
import sys
sys.path.append('../utils')
import datasets
from experiment import Experiment


XRSL_TEMPLATE = """&
(executable = "run.sh")
(inputFiles =
    ("run.sh" "")
    ("clus.jar" "../../../../../clus.jar")
    ("augmenter.py" "../../../../../../utils/augmenter.py")
    ("hierarchy.py" "../../../../../../utils/hierarchy.py")
    ("arff_manipulation.py" "../../../../../../utils/arff_manipulation.py")
    ("data.arff" "../../../../../../data/{0}")
    ("embedding.npy" "../../../../../../data/{1}")
    ("hierarchy.labels" "../../../../../../data/{2}")
    ("hierarchy.edges" "../../../../../../data/{3}")
    ("settings.s" "")
)
(outputFiles =
    ("settings.out" "")
)
(jobname = "{4}")
(cache = "no")
(count = 4)
(countpernode = 4)
(memory = 14000)
(runtimeenvironment="APPS/BASE/XENIAL-E8")
(cputime = "330 hours")
(join = "yes")
(stdout = "{4}.log")
"""


SETTINGS_TEMPLATE = """
[General]
Verbose = 0
RandomSeed = 42

[Hierarchical]
Type = {0}
WType = ExpAvgParentWeight
HSeparator = /

[Data]
File = data.arff.fold{1}.train
TestSet = data.arff.fold{1}.test

[Attributes]
{2}

[Ensemble]
EnsembleMethod = {3}
TimeBudget = 0
Iterations = 12
Optimize = Yes
NumberOfThreads = 4
SelectRandomSubspaces = 1
EnsembleBootstrapping = Yes
"""


DATASETS = datasets.vens_datasets[:-2]
FOLDS = 10


for aggregation in ['mean', 'medoid_eucl', 'medoid_hyper']:

    experiments = [
        Experiment(clustering='PCT_standard', name='standard', aggregation='None'),
        Experiment(clustering='PCT_random', name='random', method='ExtraTrees', aggregation='None')
    ]

    for dim in [2, 5, 10, 25, 50, 100]:
        for epochs in [100]:
            experiments.append(Experiment(clustering='nickel_d{}_i{}'.format(dim, epochs),
                                          dim=dim, epochs=epochs, name='nickel', aggregation=aggregation))

    for dset in DATASETS:
        for exp in experiments:
            for fold in range(FOLDS):
                attributes = []
                current_index = 1
                if exp.dim > 0:
                    attributes.append('Clustering = {}-{}'.format(current_index, current_index+exp.dim-1))
                    current_index += exp.dim
                if dset.num_keys > 0:
                    attributes.append('Key = {}'.format(','.join([str(x) for x in range(1, dset.num_keys+1)])))
                    current_index += dset.num_keys
                attributes.append('Descriptive = {}-{}'.format(current_index, current_index+dset.num_attributes-1))
                current_index += dset.num_attributes
                attributes.append('Target = {}'.format(current_index))

                job_path = 'Jobs_{}/{}/{}/fold{}'.format(aggregation, exp.clustering, dset.path[:dset.path.rfind('/')], fold)
                data = '{}.joined.arff'.format(dset.path)

                labels = '{}.labels'.format(dset.path)
                edges = '{}.edges'.format(dset.path)
                if exp.dim == 0:
                    embedding = labels
                else:
                    embedding = '{}.{}.npy'.format(dset.path, exp.clustering)

                subprocess.call('mkdir -p {}'.format(job_path), shell=True)
                with open('{}/settings.s'.format(job_path), 'w') as f:
                    print(SETTINGS_TEMPLATE.format(dset.hierarchy_type, fold, '\n'.join(attributes), exp.method), file=f)

                with open('{}/job.xrsl'.format(job_path), 'w') as f:
                    print(XRSL_TEMPLATE.format(data, embedding, labels, edges, job_path.replace('/', '-')), file=f)

                with open('{}/run.sh'.format(job_path), 'w') as f:
                    print('python3 augmenter.py data.arff embedding.npy hierarchy.edges hierarchy.labels {}'.format(
                        exp.aggregation), file=f)
                    print('python3 arff_manipulation.py data.arff {}'.format(fold), file=f)
                    print('java -Xmx50g -jar clus.jar -forest settings.s', file=f)
                    print('rm data.arff.*', file=f)
