import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pnd
import os

plt.rc('text', usetex=True)
MEASURE = '$AU\overline{PRC}$'
os.makedirs('Plots', exist_ok=True)

def aggregation_boxplot():
    sns.set(style='whitegrid', font_scale=1.5)
    df = pnd.concat([pnd.read_csv('results_mean.csv'),
                     pnd.read_csv('results_medoid_hyper.csv'),
                     pnd.read_csv('results_medoid_eucl.csv')])

    df['dataset'] = df['dataset'].str.replace('_', '\_')
    df['aggregation'] = df['aggregation'].str.replace('_', '\_')
    df = df[df['dataset'].isin(['cellcycle', 'eisen', 'spo', 'gasch1'])]

    sns.catplot(data=df[df['aggregation'] != 'None'], x='aggregation', y=MEASURE, #hue='aggregation',
                col='dataset', col_wrap=2, kind='box', legend=False)
    plt.savefig('Plots/boxplot.png', bbox_inches='tight', dpi=300)
    plt.close()


def scatter_plot():
    sns.set(style='white', font_scale=1.2)
    df = pnd.read_csv('results_mean.csv')
    df['dataset'] = df['dataset'].str.replace('_', '\_')
    df['clustering'] = df['clustering'].str.replace('nickel', 'embedding')
    grouped = df.groupby(['dataset', 'clustering', 'dim', 'precision', 'epochs', 'aggregation']).agg(['mean', 'std']).reset_index()

    def my_plot(*args, **kwargs):
        data = kwargs.pop('data')
        plt.errorbar(x=data['time']['mean'], y=data[MEASURE]['mean'],
                     yerr=data[MEASURE]['std'], fmt='none', **kwargs)
        plt.scatter(x=data['time']['mean'], y=data[MEASURE]['mean'], **kwargs)

    g = sns.FacetGrid(data=grouped, col='dataset', hue='clustering', sharex=False, col_wrap=2,
                      hue_kws=dict(marker=['o', 's', 'x']), legend_out=False,
                      hue_order=['embedding', 'standard', 'random'])
    g.map_dataframe(my_plot)
    handles, labels = g.axes[0].get_legend_handles_labels()
    plt.legend(handles[:3], labels[:3], loc='upper right', bbox_to_anchor=(1.7, 1))
    plt.savefig('Plots/scatter.png', bbox_inches='tight', dpi=300)
    plt.close()


def budget_plot():
    sns.set(style='white')
    df = pnd.read_csv('results_budget.csv')
    df['dataset'] = df['dataset'].str.replace('_', '\_')
    df['clustering'] = df['clustering'].str.replace('nickel', 'embedding')
    grouped = df.groupby(['dataset', 'clustering', 'checkpoint']).mean()
    df['time'] = df.apply(lambda x: grouped.loc[(x['dataset'], x['clustering'], x['checkpoint'])]['time'], axis=1)

    def my_plot(*args, **kwargs):
        sns.lineplot(x='time', y=MEASURE, hue='clustering', style='clustering', ci='sd',
                     hue_order=['embedding', 'standard'], style_order=['embedding', 'standard'], **kwargs)

    g = sns.FacetGrid(data=df, col='dataset', legend_out=True)
    g.map_dataframe(my_plot)
    g.fig.legend(handles=g._legend_data.values(), labels=g._legend_data.keys(),
                 loc='upper center', ncol=3, bbox_to_anchor=(0.5, 1.05))
    plt.savefig('Plots/budget.png', bbox_inches='tight', dpi=300)
    plt.close()


if __name__ == '__main__':
    aggregation_boxplot()
    scatter_plot()
    budget_plot()
