from torch.utils.data import Dataset
import numpy as np
import random


class HierarchyDataset(Dataset):

    def __init__(self, hierarchy):
        self.h = hierarchy
        self.positives = {}
        for node in self.h.node_order:
            self.positives[node.order] = {n.order for n in node.get_ancestors() | node.get_descendants()}

        self.num_negatives = 10
        self.size = self.h.number_nodes-1

    def __len__(self):
        return self.size

    def __getitem__(self, idx):
        n1 = idx+1
        n2 = random.sample(self.positives[n1], 1)[0]

        negatives = []
        while len(negatives) < self.num_negatives:
            s = random.randint(0, self.h.number_nodes-1)
            if s not in self.positives[n1]:
                negatives.append(s)

        return {
            'source': n1,
            'neighbours': np.array([n2]+list(negatives)),
            'targets': 0
        }
