import os
import sys
import torch
import pickle
from model import Embedding
from time import time
from optimizer import RiemannianSGD
from dataset import HierarchyDataset
from torch.utils.data import DataLoader
sys.path.append('../utils')
import datasets
from hierarchy import Hierarchy

DEVICE = 'cuda:0' if torch.cuda.is_available() else 'cpu'

DIMS = [2, 5, 10, 25, 50, 100]
EPOCHS = 100
LR = 1e-0
PRINT_INTERVAL = 100
BS = 100


for dset in datasets.vens_datasets:
    print(dset.name)
    h = Hierarchy.parse_edgelist('../data/{}.edges'.format(dset.path), '../data/{}.labels'.format(dset.path))
    h.print_stats()
    dataset = HierarchyDataset(h)
    loader = DataLoader(dataset, num_workers=1, shuffle=True, batch_size=BS)
    for dim in DIMS:
        print(50*'#')
        start_time = time()
        model = Embedding(h.number_nodes, dim).to(DEVICE)
        loss = torch.nn.CrossEntropyLoss()
        optimizer = RiemannianSGD(model.optim_params(), lr=LR)

        updates = 0
        loss_estimate = 0
        for epoch in range(1, EPOCHS+1):

            for batch in loader:
                sources = batch['source'].to(DEVICE)
                neighbours = batch['neighbours'].to(DEVICE)
                targets = batch['targets'].to(DEVICE)
                predictions = model(sources, neighbours)
                e = loss(predictions, targets)
                e.backward()
                loss_estimate += e.item()
                updates += 1
                optimizer.step()
                model.zero_grad()

                if updates % PRINT_INTERVAL == 0:
                    print('  '.join([
                        'Time: {:<10}'
                        'Epoch: {:<5}',
                        'Updates: {:<5}',
                        'Training loss: {:8.6f}',
                    ]).format(
                        (time() - start_time) // 1,
                        epoch,
                        updates,
                        loss_estimate / PRINT_INTERVAL,
                    ))
                    loss_estimate = 0

        model.save_weights('../data/{}.nickel_d{}_i{}.npy'.format(dset.path, dim, EPOCHS))

        times_path = '../data/{}.times'.format(dset.path)
        if os.path.exists(times_path):
            times = pickle.load(open(times_path, 'rb'))
        else:
            times = {}
        times['nickel_d{}_i{}.npy'.format(dim, EPOCHS)] = (time() - start_time) // 1

        with open(times_path, 'wb') as f:
            pickle.dump(times, f)
