import numpy as np
import sys
# from numba import njit
from hierarchy import Hierarchy


# @njit
def sum_square(x):
    return np.sum(np.square(x))


# @njit
def euclidean_distance(x, y):
    return np.sqrt(sum_square(x-y))


# @njit
def poincare_distance(x, y):
    return np.arccosh(1 + 2 * sum_square(x-y) / ((1 - sum_square(x)) * (1 - sum_square(y))))


def augment_dataset(dataset, embedding, edges, labels, aggregation='mean'):
    h = Hierarchy.parse_edgelist(edges, labels)
    matrix = np.load(embedding)
    dim = matrix.shape[1]
    distance_archive = {}
    label_to_idx = {}

    with open(labels) as f:
        for line in f:
            words = line.strip().split()
            label_to_idx[words[1]] = int(words[0])

    def mean_embedding(labels):
        embs = [matrix[label_to_idx[l]] for l in labels]
        return np.mean(embs, axis=0)

    def distance(l1, l2, dist):
        i1 = label_to_idx[l1]
        i2 = label_to_idx[l2]
        i1, i2 = min(i1, i2), max(i1, i2)
        if (i1, i2) in distance_archive:
            return distance_archive[(i1, i2)]
        else:
            d = dist(matrix[i1], matrix[i2])
            distance_archive[(i1, i2)] = d
            return d

    def medoid_embedding(labels, dist):
        medoid_label = 'root'
        best_dist = float('inf')
        for l in labels:
            d = np.sum([distance(l, l2, dist) for l2 in labels if l != l2])
            if d < best_dist:
                best_dist = d
                medoid_label = l
        return matrix[label_to_idx[medoid_label]]

    if aggregation == 'mean':
        aggr = mean_embedding
    elif aggregation == 'medoid_eucl':
        aggr = lambda x: medoid_embedding(x, euclidean_distance)
    elif aggregation == 'medoid_hyper':
        aggr = lambda x: medoid_embedding(x, poincare_distance)
    else:
        raise ValueError("Unknown aggregation type")

    def labels_to_latent(labels):
        vec = aggr(labels)
        return [str(x) for x in vec]

    with open(dataset) as f:
        lines = f.readlines()

    with open(dataset, 'w') as f:
        print('@relation augmented', file=f)
        for i in range(dim):
            print('@attribute latent{} numeric'.format(i + 1), file=f)

        in_data = False
        for line in lines:
            if line.lower().startswith('@attribute'):
                print(line.strip(), file=f)

            elif line.lower().startswith('@data'):
                in_data = True
                print('@data', file=f)

            elif in_data:
                words = line.strip().split(',')
                labels = set(words[-1].split('@'))

                latent = labels_to_latent(labels)
                print('{},{}'.format(','.join(latent), ','.join(words)), file=f)


if __name__ == '__main__':
    if len(sys.argv) == 6:
        augment_dataset(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    else:
        augment_dataset(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
